// @ts-nocheck
import React, { ReactElement, useEffect, useRef, useState, useMemo } from 'react';
import { useLatest, useThrottleFn } from 'ahooks';
import { isFunction, pick, noop, get, isEmpty } from 'lodash-es';
import classNames from 'classnames';
import { message, Modal, Tabs as TabPane } from 'antd';
import KeyCode from 'rc-util/lib/KeyCode';
import { observer } from 'mobx-react-lite';
import ImgView, { ImgViewProps, ImgViewRef } from './ImgView';
import VideoView, { VideoViewProps, VideoViewRef } from './VideoView';
import CompareView from './CompareView';
import Tool, { ToolProps } from './Tool';
import Header, { Tabs, TabsType, tabsFilter } from './Header';
import Navigation from './Nav';
import AttributePanel from './AttributePanel';
import { usePropState, cssPrefix, getSolt } from './utils';
import { screenshotButtonProp, MODEL_TYPE, Rect } from '../adapter/BigImageModalAdapter/interface';
import NullView, { NullDialogProp } from './NullDialog';
import styles from './index.module.scss';
import { generateImg } from '@common/utils';
import IconFont from '../iconfont/IconFont';
import addEventListenerWrap from '@common/utils/addEventListener';
import Carousel, { CarouselProps } from './Carousel';
import { doubleClick } from '@common/components/doubleCheck';
import { store } from '@store/index';
import { store as achieveStore } from '@pages/archive/store';
import { ObjectType } from '@store/Monitor';

const componentName = cssPrefix;

export type selecItem = Partial<Omit<ImgViewProps, 'screenshotButtonRender'>> &
  Partial<Omit<VideoViewProps, 'screenshotButtonRender'>>;

export interface ParamProps {
  tab: string;
  selectItem: selecItem;
  imgViewRef: React.MutableRefObject<ImgViewRef>;
  VideoViewRef: React.MutableRefObject<VideoViewRef>;
  model: MODEL_TYPE;
  setModel: React.Dispatch<React.SetStateAction<MODEL_TYPE>>;
  /* 可观测值 */
  scale$: number;
  showCrop$: boolean;
}

export interface BigImageData {
  //imageKey 小图
  extendRectList: (Rect & { algorithmVersion: AlgorithmVersionStr; imageKey: string })[];
  rectList: (Rect & { algorithmVersion: AlgorithmVersionStr; imageKey: string })[];
  attachImg: { url: string; label: '形体' | '人脸' }[];
  odRect: Rect;
  imageKey: string; //大图
  imgSummary: string; //摘要图

  objectExtImageKey: string; //比对到的目标图扩展图 === imgSummary
  attributeList: { label: string; list: any[] }[];
  archiveImages?: any;
  spaceName: string;

  objectType: ObjectType;
  objectId: string; //这张摘要本身的Id
  bodyObjectId?: string;
  faceObjectId?: string; //这张摘要下的人脸Id(如果有)
  sourceObjectId?: string; //这张摘要上游的形体Id(如果有)
  cameraId: string;
  cameraName: string;
  selectIndex: number;
  humanProperty: HumanProperty;
  qualityScore?: number; //人脸质量分
  score: number; //相似度
  timestamp: string;
  bodyImageUrl: string;
  faceImageUrl: string;
  algorithmVersion: AlgorithmVersionStr;

  bodySpaceName: string;
  faceSpaceName: string;

  solutionId?: Solution;
  [index: string]: any;
}

export interface BigImageDialogProps {
  visible?: boolean;
  defaultModel?: MODEL_TYPE;
  onClose?: () => void;
  isLoading?: boolean;
  hasPre?: boolean;
  hasNext?: boolean;
  selectIndex?: number;
  onSelectIndexChange?: (index: number) => void;
  dataSource: any[];
  relatedData?: any[];
  transformPropFunc: (item: any) => selecItem;
  transformVideoPropFunc: (
    item: selecItem
  ) => Promise<Omit<VideoViewProps, 'screenshotButtonRender'>>;
  screenshotButtonRender?: (screenshotButtonProp: screenshotButtonProp) => ReactElement;
  showTool?: boolean;
  showCarousel?: boolean;
  imgViewProp?: Partial<ImgViewProps>;
  videoViewProp?: Partial<VideoViewProps>;
  ToolProps?: Partial<ToolProps>;
  nullDialogProp?: Partial<NullDialogProp>;
  showHeader?: boolean;
  tabsFilter?: tabsFilter;
  useVirtual?: boolean;
  loadNext?: (index: number) => Promise<void>;
  loadPre?: (index: number) => Promise<void>;
  children: React.ReactNode;
  title?: string;
  specialTitle?: string;
  isRelated?: boolean;
  carouselProp?: Partial<CarouselProps>;
}

//查看对比图和场景图组件
const BigImageDialog: React.FC<BigImageDialogProps> = (props) => {
  const {
    hasPre: propHasPre = false,
    hasNext: propHasNext = false,
    loadNext = noop,
    loadPre = noop,
    visible = true,
    onClose,
    isLoading: propIsLoading = false,
    selectIndex: propSelectIndex = 0,
    onSelectIndexChange,
    dataSource = [],
    relatedData = [],
    screenshotButtonRender,
    transformPropFunc,
    transformVideoPropFunc,
    defaultModel = 'IMAGE',
    imgViewProp: otherImgViewProp,
    videoViewProp: otherVideoViewProp,
    carouselProp: otherCarouselProp,
    ToolProps: otherToolProp,
    showTool = true,
    children,
    nullDialogProp,
    showHeader = true,
    tabsFilter = ['COMPATER', 'NORMAL'],
    useVirtual,
    title = '查看大图',
    specialTitle = '',
    showCarousel = true,
    isRelated = false,
  } = props;

  console.log('props', props)
  //可观测值
  const showCrop$Ref = useRef(false);
  const scale$Ref = useRef(0);
  const showCrop$ = showCrop$Ref.current;
  const scale$ = scale$Ref.current;
  // ========================== 头切换 =========================
  const [tab, setTab] = useState<TAB_TYPE>(
    tabsFilter.length > 1 ? (Object.keys(Tabs)[0] as TAB_TYPE) : (Object.keys(Tabs)[1] as TAB_TYPE)
  );
  const [activeKey, setActiveKey] = useState<string>('related');
  const [similarList, setSimilarList] = useState<any>(dataSource); // 其他相似目标列表
  const [relatedList, setRelatedList] = useState<any>(relatedData); // 已关联列表
  const [imageData, setImageData] = useState<any>(dataSource); // 其他相似目标列表
  const lastSimData = useLatest(similarList);
  const lastRelatedData = useLatest(relatedList);

  // ========================== 内部转换 =========================
  const [model, setModel] = useState<MODEL_TYPE>(defaultModel); // 图片/视频模式
  useEffect(() => {
    if (activeKey === 'unRlated') {
      setImageData(lastSimData.current);
      setSelectIndex(0);
    }
    if (activeKey === 'related') {
      setImageData(lastRelatedData.current);
      setSelectIndex(0);
    }
  }, [activeKey]);
  useEffect(() => {
    setSimilarList(dataSource);
    if (activeKey === 'unRelated') {
      setImageData(dataSource);
    }
  }, [dataSource]);

  const carouseDataSource = useMemo(() => {
    return imageData.map((v: any, index: number) => {
      return {
        url: generateImg(get(v, 'passerbyImage.objectRects.0.fileKey')),
        key: index,
        passerbyId: get(v, 'passerbyId'),
      };
    });
  }, [imageData, dataSource]);

  //选择
  const [selectIndex, setSelectIndex] = usePropState(propSelectIndex);
  const hasPre = propHasPre ? true : selectIndex > 0;
  const changeIndex = isRelated ? imageData.length - 1 : dataSource.length - 1;
  const hasNext = propHasNext ? true : selectIndex < changeIndex;
  //设置选中
  const latestSelectIndex = useLatest(selectIndex);
  const latestDataSource = isRelated ? useLatest(imageData) : useLatest(dataSource);
  //加一个防背推
  const seq = useRef(0);
  const { run: setSelectIndexWrapper } = useThrottleFn(
    async (arg: ((num: number) => void) | number) => {
      const _seq = ++seq.current;
      const selectIndex = latestSelectIndex.current;
      const dataSource = latestDataSource.current;
      let newIndex = isFunction(arg) ? arg(selectIndex) : arg;
      if (newIndex > dataSource.length - 1) {
        if (propHasNext) {
          await loadNext(newIndex);
        } else {
          newIndex = dataSource.length - 1;
        }
      }
      if (newIndex < 0) {
        newIndex = 0;
      }
      if (_seq !== seq.current) {
        return;
      }

      onSelectIndexChange && onSelectIndexChange(newIndex);
      setSelectIndex(newIndex);
    },
    { wait: 500 }
  );

  //异步获取数据
  const [selectItem, setSelectItem] = useState(null);
  const [isFetching, setIsFetching] = useState(true);
  //人员属性
  const [attributeList, setAttributeList] = useState([]);
  const isLoading = useMemo(() => propIsLoading || isFetching, [propIsLoading, isFetching]);
  useEffect(() => {
    (async () => {
      setIsFetching(true);
      let data: any = {};
      if (isRelated) {
        data = await transformPropFunc(imageData[selectIndex]);
      } else {
        data = await transformPropFunc(dataSource[selectIndex]);
      }
      setAttributeList(get(data, 'attributeList', []));
      //防背推
      if (selectIndex === latestSelectIndex.current) {
        setSelectItem(data);
        setIsFetching(false);
      }
    })();
  }, [selectIndex, dataSource, imageData, isRelated]);

  const selectqualityScore = useMemo(() => {
    return get(selectItem, 'qualityScore');
  }, [selectItem]);
  //支持视频url异步获取
  const [flvUrl, setFlvUrl] = useState(null);
  const [maxDuration, setMaxDuration] = useState(null);
  useEffect(() => {
    (async () => {
      if (model === 'IMAGE') {
        setFlvUrl(null);
        setMaxDuration(null);
        return;
      }

      if (isFunction(transformVideoPropFunc)) {
        //先情空
        setFlvUrl(null);
        setMaxDuration(null);
        const { flvUrl, maxDuration } = await transformVideoPropFunc(selectItem);
        //防背推
        if (selectIndex === latestSelectIndex.current) {
          setMaxDuration(maxDuration);
          setFlvUrl(flvUrl);
        }
      } else {
        setFlvUrl(get(selectItem, 'flvUrl'));
        setMaxDuration(get(selectItem, 'maxDuration'));
      }
    })();
  }, [selectItem, model]);

  //切换选中index时候变更key 让组件走销毁逻辑 减少大图/视频组件内部逻辑复杂度
  const key = 0
  const imgCmpKey = useMemo(() => `img-key-${key}`, [key]);
  const videoCmpKey = useMemo(() => `video-key-${key}`, [key]);

  // ========================== 分离参数 =========================
  const CompareViewProp = {
    targertImg: generateImg(get(selectItem, 'imgSummary', '')), // 对比图
    score: get(selectItem, 'score'),
    similarScore: get(selectItem, 'similarScore'),
    sourceImg:
      get(selectItem, 'constractKey', '') ||
      get(selectItem, 'compaterImages.0') ||
      generateImg(get(selectItem, 'imgSummary', '')),
    compaterImages: get(selectItem, 'compaterImages', []),
    specialTitle,
    isRelated,
  };
  const carouselProp = {
    selectIndex,
    setSelectIndex: setSelectIndexWrapper,
    hasPre,
    hasNext,
    dataSource: carouseDataSource,
    ...otherCarouselProp,
  } as CarouselProps;
  const imgViewProp = {
    ...pick(selectItem, ['imageKey', 'odRect', 'attachImg', 'score']),
    screenshotButtonRender,
    ...otherImgViewProp,
    showCrop$,
    scale$,
  } as ImgViewProps;

  const videoViewProp = {
    flvUrl,
    maxDuration,
    screenshotButtonRender,
    ...otherVideoViewProp,
    showCrop$,
  } as VideoViewProps;

  const ToolProps = {
    ...otherToolProp,
    isGray: true,
  } as ToolProps;

  // ========================== 插槽参数 =========================
  const imgViewRef = useRef<ImgViewRef>(null);
  const VideoViewRef = useRef<VideoViewRef>(null);
  scale$;
  const param: ParamProps = {
    tab,
    selectItem,
    imgViewRef,
    VideoViewRef,
    model,
    setModel,
    showCrop$,
    scale$,
  };
  const slots = getSolt(children, param);

  // ========================== 监听键盘 =========================
  useEffect(() => {
    const onKeyDown = (event) => {
      const { keyCode } = event;
      if (keyCode === KeyCode.LEFT) {
        setSelectIndexWrapper((pre) => --pre);
      }
      if (keyCode === KeyCode.RIGHT) {
        setSelectIndexWrapper((pre) => ++pre);
      }
    };
    document.addEventListener('keydown', onKeyDown);
    return () => {
      document.removeEventListener('keydownm', onKeyDown);
    };
  }, []);

  // ========================== 监听滚轮 =========================
  React.useEffect(() => {
    const mousewheelListener = (e) => {
      e.preventDefault();
      let direction = 0;
      if (e.wheelDelta) {
        //IE/Opera/Chrome
        if (e.wheelDelta > 0) {
          //向上滚动事件
          direction = 1;
        } else {
          //向下滚动事件
          direction = -1;
        }
      } else if (e.detail) {
        //Firefox
        if (e.detail < 0) {
          //向上滚动事件
          direction = 1;
        } else {
          //向下滚动事件
          direction = -1;
        }
      }
      setSelectIndexWrapper((pre) => {
        return direction < 0 ? pre + 1 : pre - 1;
      });
    };
    let handleMouseWheel;
    if (store.bigImgScrollEmit) {
      handleMouseWheel = addEventListenerWrap(document, 'mousewheel', mousewheelListener);
    } else {
      handleMouseWheel?.remove();
    }
    return () => {
      handleMouseWheel?.remove();
    };
  }, [store.bigImgScrollEmit]);

  const tabList = [
    {
      key: 'related',
      label: '关联的目标',
      children: (
        <div className={styles.relatedContent}>
          {showCarousel && (
            <div className={classNames(styles.relatedPics)}>
              <Carousel {...carouselProp} />
            </div>
          )}
          <div
            style={
              isEmpty(relatedList) ? { color: 'rgba(0, 0, 0, 0.25)', cursor: 'not-allowed' } : {}
            }
            className={classNames(styles.relatedBtn)}
            onClick={async (e) => {
              if (isEmpty(relatedList)) {
                e.stopPropagation();
              } else {
                try {
                  const { archiveId, passerbyId } = selectItem;
                  if (archiveId && passerbyId) {
                    doubleClick({
                      title: '确认取消关联，取消后可在“其他相似目标推荐”处再次关联',
                      checkboxLabel: '同时移除档案对应轨迹',
                      checked: true,
                      onOk: async () => {
                        try {
                          await achieveStore.archive.UnLinkTarget(passerbyId, archiveId);
                          message.success('目标已成功取消关联！');
                          // 更新已关联目标列表
                          const { relatedPassersby } = await achieveStore.archive.GetRelatedArchive(
                            archiveId
                          );
                          setRelatedList(relatedPassersby);
                          setImageData(relatedPassersby);
                          // 更新当前相似目标列表
                          const { similarPassersby } =
                            await achieveStore.archive.GetSimilarPasserbyList(archiveId);
                          setSimilarList(similarPassersby);
                          setSelectIndex((pre) => {
                            if (pre > 0) {
                              return pre - 1;
                            } else {
                              return 0;
                            }
                          });
                        } catch (e) {
                          message.success('目标取消关联失败！');
                          console.error(e);
                        }
                      },
                    });
                  }
                } catch (e) {
                  console.error(e);
                }
              }
            }}
          >
            <IconFont
              icon="icon-guanlian"
              styles={{ fontWeight: 'bold', margin: '0px 4px 0px 17px' }}
              size={16}
            />
            取消关联
          </div>
        </div>
      ),
    },
    {
      key: 'unRlated',
      label: '其他相似目标推荐',
      children: (
        <div className={styles.relatedContent}>
          {showCarousel && (
            <div className={classNames(styles.relatedPics)}>
              <Carousel {...carouselProp} />
            </div>
          )}

          <div
            style={
              isEmpty(lastSimData.current)
                ? { color: 'rgba(0, 0, 0, 0.25)', cursor: 'not-allowed' }
                : {}
            }
            className={classNames(styles.relatedBtn)}
            onClick={async (e) => {
              if (isEmpty(lastSimData.current)) {
                e.stopPropagation();
              } else {
                try {
                  const { archiveId, passerbyId, archiveGroupId, trajectoryId } = selectItem;

                  if (archiveId && passerbyId && archiveGroupId && trajectoryId) {
                    await achieveStore.archive.LinkTarget({
                      passerbyId,
                      archiveId,
                      archiveGroupId,
                      trajectoryId,
                    });
                    message.success('目标已成功关联！');
                    // 更新当前相似目标列表
                    const { similarPassersby } = await achieveStore.archive.GetSimilarPasserbyList(
                      archiveId
                    );
                    setSimilarList(similarPassersby);
                    setImageData(similarPassersby);

                    // 更新已关联目标列表
                    const { relatedPassersby } = await achieveStore.archive.GetRelatedArchive(
                      archiveId
                    );
                    setRelatedList(relatedPassersby);
                    setSelectIndex((pre) => {
                      if (pre > 0) {
                        return pre - 1;
                      } else {
                        return 0;
                      }
                    });
                  }
                } catch (e) {
                  message.error('目标关联失败！');
                  console.error(e);
                }
              }
            }}
          >
            <IconFont
              icon="icon-guanlian"
              styles={{ fontWeight: 'bold', margin: '0px 4px 0px 17px' }}
              size={16}
            />
            关联
          </div>
        </div>
      ),
    },
  ];
  return (
    <Modal
      open={visible}
      onCancel={onClose}
      footer={null}
      className={classNames(componentName)}
      centered
      title={title}
    >
      {!isEmpty(attributeList) && <AttributePanel attributeList={attributeList} />}
      {showHeader && (
        <Header
          noBgColor={!isEmpty(attributeList)}
          value={tab}
          onChange={setTab}
          tabsFilter={tabsFilter}
        />
      )}
      <div
        className={classNames(`${componentName}-view-container`)}
        style={
          tab === 'TRACK'
            ? {
                height: '718px',
                marginBottom: '0px',
              }
            : {}
        }
      >
        {tab !== 'TRACK' && (
          <>
            {/* ---------------------------------- 对比图模式 --------------------------------- */}
            {model == 'IMAGE' && tab === 'COMPATER' && !isEmpty(selectItem) && (
              <CompareView {...CompareViewProp} />
            )}
            {/* ---------------------------------- 场景图模式 --------------------------------- */}
            {!isLoading &&
              model == 'IMAGE' &&
              tab === 'NORMAL' &&
              !isEmpty(selectItem?.rectList) && (
                <ImgView key={imgCmpKey} {...imgViewProp} ref={imgViewRef} />
              )}
            {/* -------------------------------- 暂无图片数据 -------------------------------- */}
            {!isLoading && model == 'IMAGE' && isEmpty(selectItem) && tab === 'COMPATER' && (
              <NullView {...nullDialogProp}></NullView>
            )}
            {!isLoading && model == 'IMAGE' && isEmpty(selectItem.rectList) && tab === 'NORMAL' && (
              <NullView emptyText={isRelated ? '暂无匹配数据' : '暂无数据'}></NullView>
            )}
            {/* --------------------------------- 视频播放模式 --------------------------------- */}
            {!isLoading && model == 'VIDEO' && (
              <VideoView key={videoCmpKey} {...videoViewProp} ref={VideoViewRef} />
            )}
            {/* ----------------------------------- 左导航 ---------------------------------- */}
            <Navigation
              className={classNames(
                `${componentName}-view-container__nav`,
                !hasPre && `${componentName}-view-container__nav--disabled`,
                `${componentName}-view-container__nav--left`
              )}
              show={isRelated ? imageData.length !== 1 : dataSource.length !== 1}
              disabled={!hasPre}
              prev
              onClick={(e) => {
                if (useVirtual && selectIndex === 0) {
                  !!loadPre && loadPre();
                  //都跳转到第一个
                  setSelectIndexWrapper(0);
                } else {
                  setSelectIndexWrapper((pre) => pre - 1);
                }
              }}
            />
            {/* ----------------------------------- 右导航 ---------------------------------- */}
            <Navigation
              className={classNames(
                `${componentName}-view-container__nav`,
                !hasNext && `${componentName}-view-container__nav--disabled`,
                `${componentName}-view-container__nav--right`
              )}
              show={isRelated ? imageData.length !== 1 : dataSource.length !== 1}
              disabled={!hasNext}
              next
              onClick={async (e) => {
                if (isRelated) {
                  if (useVirtual && selectIndex === imageData.length - 1) {
                    !!loadNext && (await loadNext());
                    //都跳转到第一个
                    setSelectIndexWrapper(0);
                  } else {
                    setSelectIndexWrapper((pre) => pre + 1);
                  }
                } else {
                  if (useVirtual && selectIndex === dataSource.length - 1) {
                    !!loadNext && (await loadNext());
                    //都跳转到第一个
                    setSelectIndexWrapper(0);
                  } else {
                    setSelectIndexWrapper((pre) => pre + 1);
                  }
                }
              }}
            />
          </>
        )}
      </div>
      {/* 工具集合 */}
      {tab !== 'TRACK' && showTool && (
        <>
          <div className={classNames(`${componentName}-tool-container`)}>
            <Tool {...ToolProps} param={param} />
          </div>
          {slots['after-tool']}
        </>
      )}
      {/* 人脸质量分 */}
      {!!selectqualityScore && store.user.isfaceScore && model == 'IMAGE' && tab === 'NORMAL' && (
        <div
          style={isRelated ? { bottom: 220 } : { bottom: 90 }}
          slot={'after-tool'}
          className={classNames(`${componentName}__face-score`)}
        >{`人脸质量分：${(selectqualityScore as number).toFixed(2)}`}</div>
      )}
      {/* --------------------------------- 模型碰撞组件 --------------------------------- */}
      {isRelated && tab !== 'TRACK' && (
        <div className={styles.relatedWrapper}>
          <TabPane
            className={styles.relatedTabs}
            tabPosition={'left'}
            activeKey={activeKey}
            onChange={(key: string) => {
              setActiveKey(key);
            }}
            items={tabList.map((item) => {
              return {
                label: item.label,
                key: item.key,
                children: item.children,
              };
            })}
          />
        </div>
      )}
    </Modal>
  );
};

export default observer(BigImageDialog);
