// @ts-nocheck
import { v4 as uuidv4 } from 'uuid';
import { has } from '@zhst/func';
import io from 'socket.io-client';
import { SOCKET_HOST } from '@common/constants';

const EMITSTATE = {
  NOT_CONNECT: 0,
  WAITING: 1,
  CONNECT: 2,
};

const initRetryTime = 0;
const intervalTime = 5 * 1000; //下次重试增加时间
const maxIntervalTime = 1 * 60 * 60 * 1000; //最大重试时间1小时

class Channel {
  /**
   * io 实例化对象
   */
  ioIns;
  /**
   * 已存在的订阅列表
   */
  listeners = [
    // {
    //     topic: "",
    //     req: "",
    //     suInfo: {},
    //     hasEmit: false,//是否已经订阅
    //     lastRetryInterval: 0,
    //     handles: {
    //     }
    // }
  ];

  /**
   * 调试信息 记录订阅/反订阅次数
   */
  subscribeListenerId = [];
  unSubscribeListenerId = [];

  init = () => {
    const ioIns = (this.ioIns = io(SOCKET_HOST, {
      reconnection: true,
      transports: ['websocket'],
      forceNew: true,
    }));
    ioIns.on('connect', (...arg: any) => {
      console.debug('connect', arg);
      this.ioIns = ioIns;
      this.listeners.forEach((v) => {
        this.doEmit(v['topic'], v['req'], v['id']);
      });
    });

    ioIns.on('event', (...arg: any) => {
      console.debug('event', arg);
    });

    ioIns.on('disconnect', (...arg: any) => {
      console.debug('disconnect', arg);
      this.subscribeListenerId = [];
      this.unSubscribeListenerId = [];
    });

    ioIns.on('reconnect', (...arg: any) => {
      console.debug('reconnect', arg);
      this.listeners.forEach((v) => {
        v['hasEmit'] = EMITSTATE.NOT_CONNECT;
        this.doEmit(v['topic'], v['req'], v['id']);
      });
    });
  };

  retry = (listener: { [x: string]: any; lastRetryInterval: number | undefined; intervalId: NodeJS.Timeout; } | undefined) => {
    //重试逻辑
    const intervalId = setTimeout(() => {
      const hasExit = this.listeners.find(
        (v) => v['topic'] === listener?.['topic'] && v['req'] === listener['req']
      );
      if (!hasExit) return;
      listener['hasEmit'] = EMITSTATE.NOT_CONNECT;
      this.doEmit(listener['topic'], listener['req'], listener['id']);
    }, listener.lastRetryInterval);

    listener.intervalId = intervalId;
    listener.lastRetryInterval =
      intervalTime + listener.lastRetryInterval > maxIntervalTime
        ? maxIntervalTime
        : intervalTime + listener.lastRetryInterval;
  };

  doEmit = (topic, req, listenerId) => {
    if (!this.ioIns) {
      this.init();
    }
    //订阅过就不订阅了
    const hasEmit = this.listeners.find(
      (v) => v['topic'] === topic && v['req'] === req && v['hasEmit'] !== EMITSTATE.NOT_CONNECT
    );
    if (hasEmit) {
      return;
    }
    const listener = this.listeners.find((v) => v['topic'] === topic && v['req'] === req);
    listener['hasEmit'] = EMITSTATE.WAITING;

    this.ioIns?.emit?.(topic, req, (data) => {
      console.info('emit', topic, req, data);
      const suInfo = JSON.parse(data);
      if (has(suInfo, 'Error.code')) {
        if (suInfo.Error.code === 500) {
          //后端出错
          this.retry(listener);
        }
        return;
      }
      // console.debug('SUBSCRIBE', listenerId, topic, req, suInfo);
      this.subscribeListenerId.push(listenerId);
      // debugger
      //重新找一遍topic
      const currentTopicIndex = this.listeners.findIndex(
        (v) => v['topic'] === topic && v['req'] === req && v['id'] === listenerId
      );
      if (currentTopicIndex == -1) {
        // 不存在说明listener取消了 直接反订阅
        this.ioIns.emit('UnSubscribe', JSON.stringify(suInfo), (data) => {
          this.unSubscribeListenerId.push(listenerId);
          console.debug('UNSUBSCRIBE', listenerId, topic, req, data);
        });
        return;
      }
      if (!suInfo['SubscribeID']) {
        this.listeners.splice(currentTopicIndex, 0);
      } else {
        this.listeners[currentTopicIndex]['suInfo'] = suInfo;
        this.listeners[currentTopicIndex]['hasEmit'] = EMITSTATE.CONNECT;
      }

      this.ioIns?.on?.(suInfo['SubscribeID'], (data) => {
        console.info('on', suInfo['SubscribeID'], data);
        try {
          const socketData = JSON.parse(data);
          if (has(socketData, 'Error.code')) {
            if (socketData.Error.code === 500) {
              //后端出错
              this.retry(listener);
            }
            return;
          }
          const { handles = {} } =
            this.listeners.find((v) => v['topic'] === topic && v['req'] === req) || {};
          Object.keys(handles).forEach((key) => {
            try {
              //后面观察 为什么delete后在foreach
              const func = handles[key];
              if (!func) return;
              func(socketData);
            } catch (error) {
              console.error(error);
            }
          });
        } catch (error) {
          console.debug('error', error);
        }
      });
    });
  };

  subscribe(topic, req, handle) {
    const handleId = uuidv4();
    const listenerId = uuidv4();
    const listener = this.listeners.find((v) => v['topic'] === topic && v['req'] === req);
    if (listener) {
      listener['handles'][handleId] = handle;
    } else {
      this.listeners.push({
        topic: topic,
        req: req,
        suInfo: {},
        id: listenerId,
        hasEmit: EMITSTATE.NOT_CONNECT,
        lastRetryInterval: initRetryTime,
        handles: {
          [`${handleId}`]: handle,
        },
      });
      //未注册过 则去订阅
      this.doEmit(topic, req, listenerId);
    }
    return this.unSubscribe.bind(this, topic, req, handleId, listenerId);
  }

  unSubscribe(topic, req, handleId, listenerId) {
    const listener = this.listeners.find(
      (v) => v['topic'] === topic && v['req'] === req && v['id'] === listenerId
    );
    const { handles = {}, suInfo } = listener || {};
    if (handles[handleId]) {
      delete handles[handleId];
      //如果没有其他订阅就删除
      if (Object.keys(handles).length === 0) {
        if (this.intervalId) {
          clearTimeout(this.intervalId);
        }
        if (listener['hasEmit'] === EMITSTATE['CONNECT']) {
          this.ioIns.emit('UnSubscribe', JSON.stringify(suInfo), (data) => {
            this.unSubscribeListenerId.push(listenerId);
            console.debug('UNSUBSCRIBE', listener['id'], topic, req, data);
          });
        }
        this.listeners = this.listeners.filter((v) => !(v['topic'] === topic && v['req'] === req));
      }
    }
  }
}
//单例
const channelIns = new Channel();
//暴露实例 调试用
window.__channel__ = channelIns;

export default channelIns;
